import React, {Component} from 'react'
import PropTypes from 'prop-types'
import { is, fromJS } from 'immutable'
import { Form, Row, Col, Button, notification, Modal } from 'antd'
import { CloseOutlined } from '@ant-design/icons'
import moment from 'moment'

import Api from '@/api'
import options from '@/store/options.js'
import asyncComponent from '@/utils/asyncComponent'
import asyncSpinComponent from '@/utils/asyncSpinComponent'
import Utils from '@/utils/utils.js'
import zhCN from '@/locales/zh-CN/main.js'
import enUS from '@/locales/en-US/main.js'
import MKInput from './mkInput'
import './index.scss'

const MutilForm = asyncSpinComponent(() => import('./advanceform'))
const MKCheckCard = asyncComponent(() => import('@/tabviews/zshare/mutilform/mkCheckCard'))
const MKSelect = asyncComponent(() => import('./mkSelect'))
const DateGroup = asyncComponent(() => import('./dategroup'))
const MKDatePicker = asyncComponent(() => import('./mkDatePicker'))

class MainSearch extends Component {
  static propTpyes = {
    BID: PropTypes.any,          // 父级Id，用于查询下拉选择项
    menuType: PropTypes.any,     // 菜单权限，是否为HS
    searchlist: PropTypes.array, // 搜索条件列表
    config: PropTypes.object,    // 组件配置信息(自定义页面)
    setting: PropTypes.object,   // 组件配置信息(自定义页面)
    refreshdata: PropTypes.func  // 刷新数据
  }

  state = {
    dict: sessionStorage.getItem('lang') !== 'en-US' ? zhCN : enUS,
    searchlist: null,        // 搜索项
    reset: true,             // 控制组合搜索项重置
    float: '',               // 浮动
    showButton: true,        // 是否显示搜索按钮
    showAdvanced: false,     // 是否显示高级搜索
    searchStyle: null,       // 搜索条件样式
    advanceValues: [],       // 高级搜索条件保存值
    visible: false,
    adModelWidth: '1000px',
    hasReqFields: false
  }

  record = {}

  UNSAFE_componentWillMount () {
    const { config, menuType, searchlist, setting } = this.props

    let _searchlist = []
    let fieldMap = new Map()
    let mainItems = []  // 云端或单点数据
    let localItems = [] // 本地数据
    let deForms = []    // 测试系统，单个请求
    let float = ''
    let showButton = true
    let searchStyle = null
    let advanceValues = []
    let showAdvanced = false
    let adModelWidth = 1000
    let linkFields = {}
    let record = {}
    let hasReqFields = false

    if (setting && setting.advanceWidth) {
      adModelWidth = setting.advanceWidth
    } else if (config && config.wrap && config.wrap.advanceWidth) {
      adModelWidth = config.wrap.advanceWidth
    }

    if (adModelWidth < 100) {
      adModelWidth = adModelWidth + 'vw'
    } else {
      adModelWidth = adModelWidth + 'px'
    }

    if (searchlist) {
      if (setting && setting.show === 'false') {
        showButton = false
      }
      _searchlist = fromJS(searchlist).toJS()
    } else if (config) {
      _searchlist = fromJS(config.search).toJS()
      if (config.type === 'search' && config.subtype === 'mainsearch') {
        float = config.wrap.float
        showButton = config.wrap.show !== 'false'
        searchStyle = config.style
      } else {
        showButton = false
        float = 'right'
      }
    }

    _searchlist.forEach(item => {
      // if (item.type === 'link') {
      if (item.linkField) {
        linkFields[item.linkField] = linkFields[item.linkField] || []
        linkFields[item.linkField].push({field: item.field, uuid: item.uuid})
      }

      let labelwidth = item.labelwidth || 33.3
      if (item.labelShow === 'false') {
        labelwidth = 0
      }
      item.labelCol = {style: {width: labelwidth + '%'}}
      item.wrapperCol = {style: {width: (100 - labelwidth) + '%'}}

      if (fieldMap.has(item.field)) {
        item.field = item.field + '@tail@'
      }

      if (item.required) {
        hasReqFields = true
      }

      if (showButton && item.advanced) {
        showAdvanced = true
      } else {
        item.advanced = false
      }

      if (item.advanced && item.initval) {
        let val = item.initval
        if (item.precision === 'hour') {
          if (/,/ig.test(val)) {
            val = val.split(',').map(m => m + ':00').join(',')
          } else {
            val = val + ':00'
          }
        }
        advanceValues.push({field: item.field, type: item.type, label: item.label, value: val})
      }

      if (item.type === 'group') {
        record[item.field] = item.initType
        record[item.datefield] = item.initval
      } else {
        record[item.field] = item.initval
      }
      
      if (['select', 'link', 'multiselect', 'checkcard'].includes(item.type)) {
        item.options = item.options || []
        item.options = item.options.filter(op => !op.Hide)
        if (item.setAll === 'true' && ['select', 'link'].includes(item.type)) {
          item.options.unshift({
            key: Utils.getuuid(),
            Value: '',
            Text: '全部',
            ParentID: ''
          })
        }

        // 数据源查询语句
        if (item.resourceType === '1' && item.dataSource) {
          let _option = Utils.getSelectQueryOptions(item)

          // 测试系统单个请求
          if (menuType !== 'HS' && options.sysType === 'local' && !window.GLOB.systemType) {
            deForms.push({
              ...item,
              arr_field: _option.field,
              data_sql: Utils.formatOptions(_option.sql)
            })
          } else { // 合并请求，区分本地及系统
            if (item.database === 'sso') {
              mainItems.push(`select '${item.field}' as obj_name,'${_option.field}' as arr_field,'${window.btoa(window.encodeURIComponent(_option.sql))}' as LText`)
            } else {
              localItems.push(`select '${item.field}' as obj_name,'${_option.field}' as arr_field,'${window.btoa(window.encodeURIComponent(_option.sql))}' as LText`)
            }
          }
        }
        item.oriOptions = fromJS(item.options).toJS()
      }

      fieldMap.set(item.field, item)
    })

    this.record = record

    let _list = _searchlist.map(item => {
      if (item.hidden) return item

      if (linkFields[item.field] && (item.type === 'select' || item.type === 'link' || (item.type === 'checkcard' && item.multiple !== 'true'))) {
        item.linkFields = linkFields[item.field]
      }

      if (item.linkField) {
        let supItem = fieldMap.get(item.linkField)
        
        if (!supItem) {
          notification.warning({
            top: 92,
            message: '未查询到搜索条件《' + item.label + '》关联字段！',
            duration: 5
          })
          item.type = 'select'
        } else {
          item.supInitVal = supItem.initval
          item.options = item.oriOptions.filter(option => option.ParentID === supItem.initval || option.Value === '')
        }
      }

      return item
    })

    this.setState({
      float,
      showButton,
      searchStyle,
      hasReqFields,
      showAdvanced,
      adModelWidth,
      advanceValues,
      searchlist: _list
    }, () => {
      if (menuType !== 'HS' && options.sysType === 'local' && !window.GLOB.systemType) {
        this.improveSimpleSearch(deForms)
      } else {
        this.improveSearch(mainItems, localItems)
      }
    })
  }

  shouldComponentUpdate (nextProps, nextState) {
    return !is(fromJS(this.state), fromJS(nextState))
  }

  // 查询下拉菜单
  improveSearch = (mainItems, localItems) => {
    const { menuType, BID } = this.props
    let deffers = []

    if (menuType !== 'HS' && window.GLOB.systemType !== 'production') {
      localItems = [...localItems, ...mainItems]
      mainItems = []
    }

    // 本地请求
    let param = {
      func: 'sPC_Get_SelectedList',
      LText: localItems.join(' union all '),
      obj_name: '',
      arr_field: '',
      table_type: 'Y'
    }

    if (BID) {
      param.BID = BID
    }
    
    if (param.LText) {
      param.LText = Utils.formatOptions(param.LText)
      param.timestamp = moment().format('YYYY-MM-DD HH:mm:ss')
      param.secretkey = Utils.encrypt(param.LText, param.timestamp)

      if (menuType === 'HS') { // 云端数据验证
        param.open_key = Utils.encryptOpenKey(param.secretkey, param.timestamp)
      }

      deffers.push(
        new Promise(resolve => {
          Api.getSystemCacheConfig(param).then(res => {
            if (!res.status) {
              notification.warning({
                top: 92,
                message: res.message,
                duration: 5
              })
            }
            resolve(res)
          })
        })
      )
    }

    // 系统请求
    let mainparam = {
      func: 'sPC_Get_SelectedList',
      LText: mainItems.join(' union all '),
      obj_name: '',
      arr_field: '',
      table_type: 'Y'
    }

    if (BID) {
      mainparam.BID = BID
    }

    if (mainparam.LText) {
      mainparam.LText = Utils.formatOptions(mainparam.LText)
      mainparam.timestamp = moment().format('YYYY-MM-DD HH:mm:ss')
      mainparam.secretkey = Utils.encrypt(mainparam.LText, mainparam.timestamp)

      if (menuType === 'HS') { // 云端数据验证
        mainparam.open_key = Utils.encryptOpenKey(mainparam.secretkey, mainparam.timestamp)
        if (options.cloudServiceApi) {
          mainparam.rduri = options.cloudServiceApi
          mainparam.userid = sessionStorage.getItem('CloudUserID') || ''
          mainparam.LoginUID = sessionStorage.getItem('CloudLoginUID') || ''
        }
      } else if (window.GLOB.mainSystemApi) {
        mainparam.rduri = window.GLOB.mainSystemApi
      }

      deffers.push(
        new Promise(resolve => {
          Api.getSystemCacheConfig(mainparam).then(res => {
            if (!res.status) {
              notification.warning({
                top: 92,
                message: res.message,
                duration: 5
              })
            }
            resolve(res)
          })
        })
      )
    }

    Promise.all(deffers).then(response => {
      let result = {...response[0], ...(response[1] || {})}

      delete result.ErrCode
      delete result.ErrMesg
      delete result.message
      delete result.status

      this.resetSearch(result)
    })
  }

  // 测试系统单个请求下拉选项
  improveSimpleSearch = (deForms) => {
    if (deForms.length === 0) return

    let deffers = deForms.map((item, index) => {
      let param = {
        func: 'sPC_Get_SelectedList',
        LText: item.data_sql,
        obj_name: item.field,
        arr_field: item.arr_field
      }

      if (this.props.BID) {
        param.BID = this.props.BID
      }

      param.timestamp = moment().format('YYYY-MM-DD HH:mm:ss')
      param.secretkey = Utils.encrypt(param.LText, param.timestamp)

      return (
        new Promise(resolve => {
          setTimeout(() => {
            Api.getSystemCacheConfig(param).then(res => {
              if (!res.status) {
                notification.warning({
                  top: 92,
                  message: res.message,
                  duration: 5
                })
              }
              resolve(res)
            })
          }, index * 30)
        })
      )
    })

    Promise.all(deffers).then(response => {
      let result = {}

      response.forEach(res => {
        result = {...result, ...res}
      })

      delete result.ErrCode
      delete result.ErrMesg
      delete result.message
      delete result.status

      this.resetSearch(result)
    })
  }

  resetSearch = (result) => {
    let _searchlist = fromJS(this.state.searchlist).toJS().map(item => {
      if (['select', 'link', 'multiselect', 'checkcard'].includes(item.type) && result[item.field] && result[item.field].length > 0) {
        let options = []
        result[item.field].forEach(cell => {
          let _item = {
            key: Utils.getuuid()
          }

          if (item.type !== 'checkcard') {
            _item.Value = cell[item.valueField]
            _item.Text = cell[item.valueText]
            if (!_item.Text && _item.Text !== 0) {
              return
            }
          } else {
            _item.$value = cell[item.cardValField]
            _item = {..._item, ...cell}
          }

          if (item.linkField) {
            _item.ParentID = cell[item.linkField]
          }

          options.push(_item)
        })

        item.oriOptions = [...item.oriOptions, ...options]
      }

      if (item.linkField) {
        if (item.supInitVal) {
          item.options = item.oriOptions.filter(option => option.ParentID === item.supInitVal || option.Value === '')
        } else {
          item.options = item.oriOptions
        }
      } else if (item.oriOptions) {
        item.options = item.oriOptions
      }

      return item
    })

    this.setState({
      searchlist: _searchlist
    })
  }

  recordChange = (val, defer, item) => {
    this.record[item.field] = val

    if (defer) return

    if (item.linkFields) {
      setTimeout(() => {
        this.handleSubmit()
      }, 1000)
    } else {
      setTimeout(() => {
        this.handleSubmit()
      }, 10)
    }
  }

  dateGroupChange = (val, type, item) => {
    this.record[item.datefield] = val
    this.record[item.field] = type

    this.handleSubmit()
  }

  cardChange = (val, item) => {
    this.record[item.field] = val

    if (!item.linkFields) {
      setTimeout(() => {
        this.handleSubmit()
      }, 10)
    }
  }

  getFields() {
    const { getFieldDecorator } = this.props.form
    const { dict, showButton, showAdvanced, float, visible } = this.state
    const fields = []
    let lastRadio = 6

    this.state.searchlist.forEach((item, index) => {
      if (item.hidden || item.advanced) return

      const _rules = [
        {
          required: item.required,
          message: item.label + '不可为空!'
        }
      ]

      let content = null
      let className = ''
      let field = item.field
      lastRadio = item.ratio || 6

      if (item.type === 'text') {
        content = (<MKInput config={item} onInputSubmit={this.handleSubmit} />)
      } else if (item.type === 'select' || item.type === 'link' || item.type === 'multiselect') {
        content = (<MKSelect config={item} onChange={(val, defer) => this.recordChange(val, defer, item)} />)
      } else if (item.type === 'date' || item.type === 'datemonth' || item.type === 'dateweek' || item.type === 'daterange') {
        content = (<MKDatePicker config={item} onChange={(val) => this.recordChange(val, false, item)} />)
      } else if (item.type === 'group') {
        field = item.datefield
        content = <DateGroup position={index} config={item} onChange={(val, type) => this.dateGroupChange(val, type, item)} />
      } else if (item.type === 'checkcard') {
        className = 'checkcard'
        content = <MKCheckCard config={item} onChange={(val) => this.cardChange(val, item)} />
      }

      if (content) {
        fields.push(
          <Col span={item.ratio || 6} key={index}>
            <Form.Item
              className={className}
              label={item.labelShow !== 'false' ? item.label : ''}
              labelCol={item.labelCol}
              wrapperCol={item.wrapperCol}
            >
              {getFieldDecorator(field, {
                initialValue: item.initval,
                rules: _rules
              })(content)}
            </Form.Item>
          </Col>
        )
      }
    })

    if (showButton) {
      let action = (
        <Col span={lastRadio < 6 ? 6 : lastRadio} style={{ whiteSpace: 'nowrap' }} className="search-button" key="actions">
          <Form.Item label={' '} colon={false} style={{ minHeight: '40px' }}>
            <Button type="primary" onClick={this.handleSubmit}>
              {dict['main.search']}
            </Button>
            <Button style={{ marginLeft: 8 }} onClick={this.handleReset}>
              {dict['main.reset']}
            </Button>
            {showAdvanced && !visible ? <Button type="link" onClick={this.handleAdvance}>
              高级
            </Button> : null}
          </Form.Item>
        </Col>
      )
      if (float === 'right') {
        fields.unshift(action)
      } else {
        fields.push(action)
      }
    }
    
    return fields
  }

  handleAdvance = () => {
    this.setState({visible: true})
  }

  handleSubmit = () => {
    this.setState({}, () => {
      this.props.form.validateFields((err, values) => {
        if (err) return

        let { searches, error } = this.getFieldsValues(values)
  
        if (error) {
          notification.warning({
            top: 92,
            message: error,
            duration: 3
          })
          return
        } else if (this.state.hasReqFields) {
          let requireFields = searches.filter(item => item.required && item.value === '')
          if (requireFields.length > 0) {
            let labels = requireFields.map(item => item.label)
            labels = Array.from(new Set(labels))
      
            notification.warning({
              top: 92,
              message: this.state.dict['form.required.input'] + labels.join('、') + ' !',
              duration: 3
            })
            return
          }
        }

        this.props.refreshdata(searches)
      })
    })
  }

  /**
   * @description 搜索条件重置
   */
  handleReset = () => {
    let record = {}
    let advanceValues = []
    let searchlist = this.state.searchlist.map(item => {
      item.initval = item.oriInitval

      if (item.type === 'group') {
        record[item.datefield] = item.initval
        record[item.field] = item.initType
      } else {
        record[item.field] = item.initval
      }
      if (item.advanced && item.initval) {
        let val = item.initval
        if (item.precision === 'hour') {
          if (/,/ig.test(val)) {
            val = val.split(',').map(m => m + ':00').join(',')
          } else {
            val = val + ':00'
          }
        }
        advanceValues.push({field: item.field, type: item.type, label: item.label, value: val})
      }

      return item
    })

    this.record = record

    this.setState({searchlist: [], advanceValues}, () => {
      this.setState({searchlist}, () => {
        this.handleSubmit()
      })
    })
  }

  getFieldsValues = (vals) => {
    const { searchlist } = this.state
    let values = {...this.record, ...vals}
    this.record = values

    // 获取搜索条件值
    let search = []
    let error = null
    searchlist.forEach(item => {
      if (item.type === 'group') {
        search.push({
          type: 'daterange',
          key: item.datefield,
          value: values[item.datefield] || '',
          label: item.label,
          match: 'between',
          required: item.required,
          precision: 'day',
          forbid: item.$forbid
        }, {
          type: 'group',
          key: item.field,
          value: values[item.field] || '',
          label: item.label,
          match: '=',
          forbid: true,
          required: item.required
        })
      } else {
        let type = item.type
        let val = values[item.field] !== undefined ? values[item.field] : ''

        if (type === 'multiselect' || (type === 'checkcard' && item.multiple === 'true')) {
          type = 'multi'
        } else if (type === 'text' && !error) {
          if (/'/ig.test(val)) {
            error = `搜索条件${item.label}中，不可使用英文状态的单引号。`
          } else if (/(^|\s)select\s/ig.test(val)) {
            error = `搜索条件${item.label}中，不可使用关键字select。`
          } else if (/\sfrom(\s|\()/ig.test(val)) {
            error = `搜索条件${item.label}中，不可使用关键字from。`
          } else if (/(^|\s)insert\s/ig.test(val)) {
            error = `搜索条件${item.label}中，不可使用关键字insert。`
          } else if (/(^|\s)delete\s/ig.test(val)) {
            error = `搜索条件${item.label}中，不可使用关键字delete。`
          } else if (/(^|\s)update\s/ig.test(val)) {
            error = `搜索条件${item.label}中，不可使用关键字update。`
          } else if (/(^|\s)where\s/ig.test(val)) {
            error = `搜索条件${item.label}中，不可使用关键字where。`
          } else if (/(^|\s)drop\s/ig.test(val)) {
            error = `搜索条件${item.label}中，不可使用关键字drop。`
          } else if (/(^|\s)exec\s/ig.test(val)) {
            error = `搜索条件${item.label}中，不可使用关键字exec。`
          } else if (/(^|\s)truncate\s/ig.test(val)) {
            error = `搜索条件${item.label}中，不可使用关键字truncate。`
          } else if (/(^|\s)while\s/ig.test(val)) {
            error = `搜索条件${item.label}中，不可使用关键字while。`
          } else if (/(^|\s)for\s/ig.test(val)) {
            error = `搜索条件${item.label}中，不可使用关键字for。`
          }
        }

        if (typeof(val) === 'string') {
          val = val.replace(/(^\s*|\s*$)/ig, '')
        }

        search.push({
          type: type,
          key: item.field.replace(/@tail@$/, ''),
          value: val,
          label: item.label,
          match: item.match,
          required: item.required,
          precision: item.precision || 'day',
          forbid: item.$forbid
        })
      }
    })

    return {searches: search, error}
  }

  handleOk = (values) => {
    this.record = {...this.record, ...values}

    let advanceValues = []
    this.state.searchlist.forEach(item => {
      if (!item.advanced) return

      let val = this.record[item.field]
      if (val || val === 0) {
        if (item.precision === 'hour') {
          if (/,/ig.test(val)) {
            val = val.split(',').map(m => m + ':00').join(',')
          } else {
            val = val + ':00'
          }
        }
        advanceValues.push({field: item.field, type: item.type, label: item.label, value: val})
      }
    })

    this.setState({advanceValues, visible: false})
    this.handleSubmit()
  }

  closeAdvanceForm = (cell) => {
    this.record[cell.field] = ''

    this.setState({advanceValues: this.state.advanceValues.filter(item => item.field !== cell.field)})
    this.handleSubmit()
  }

  render() {
    const { float, searchStyle, visible, searchlist, showAdvanced, advanceValues, adModelWidth } = this.state
    const formItemLayout = {
      labelCol: {
        xs: { span: 24 },
        sm: { span: 8 }
      },
      wrapperCol: {
        xs: { span: 24 },
        sm: { span: 16 }
      }
    }

    return (
      <>
        <Form {...formItemLayout} className={`top-search ${float}`} style={searchStyle}>
          <Row gutter={24}>{this.getFields()}</Row>
          <Row gutter={24}>
            {showAdvanced ? <div className="advanced-list">
              {advanceValues.map((item, index) => {
                return (
                  <div key={index}>
                    <span>{item.label}: </span>
                    <span className="advance-value">{item.value}</span>
                    <CloseOutlined onClick={() => this.closeAdvanceForm(item)} />
                  </div>)
              })}
            </div> : null}
          </Row>
        </Form>
        <Modal
          title="高级搜索"
          maskClosable={false}
          visible={visible}
          width={adModelWidth}
          closable={false}
          footer={null}
          destroyOnClose
        >
          <MutilForm
            searchlist={searchlist}
            record={this.record}
            advanceSubmit={this.handleOk}
            handleClose={() => this.setState({visible: false})}
          />
        </Modal>
      </>
    )
  }
}

export default Form.create()(MainSearch)